﻿/* Set expanding and collapsing events and animations, add/remove classes and background images */
(function (Adf) {

    "use strict";

    var SingleExpanding = Adform.Component && Adform.Component.SingleExpanding;

    // REUSABLE BLOCK LANDING FUNCTION - valid for mraid formats //
    function land(options) {
        if (!options.clicktag) {
            return;
        }
        if (window.dhtml === undefined) {
            mraid.open(options.clicktag);
        } else {
            window.open(options.clicktag, options.target || '_blank');
        }
    }

    // REUSABLE UTILITIES //
    /* ADDS CLASSES TO ELEMENTS (WORKING WITH MULTIPLE CLASSES ON IE) */
    function addClass(elem, classes) {
        elem.className = elem.className.split(' ').concat(classes).filter(function(elem, index, self) {
            return index === self.indexOf(elem);
        }).join(' ');
    }

    /* REMOVES CLASSES TO ELEMENTS (WORKING WITH MULTIPLE CLASSES ON IE) */
    function removeClass(elem, classes) {
        elem.className = elem.className.split(' ').filter(function(value) {
            return classes.indexOf(value) < 0;
        }).join(' ');
    }

    /* TOGGLE CLASSES TO ELEMENT */
    function toggleClass(elem, classes) {
        var classToRemove = elem.className.split(' ').filter(function(value) {
            return classes.indexOf(value) >= 0;
        });
        addClass(elem, classes);
        removeClass(elem, classToRemove);
    }

    /* SETS IMAGE AS BACKGROUND */
    function setBackgroundImage(elem, background, position, size) {
        if (elem && background) {
		    elem.style.background = 'url("' + background + '") no-repeat right top' ;
		}
    }

    /* REMOVES ELEMENTS WHEN IMAGE IS SET AS BACKGROUND */
    function removeElems() {
        [].forEach.call(arguments, function(elem) {
            elem && elem.parentNode.removeChild(elem);
        });
    }

    // REAL CUSTOM FUNCTION //
    function Expandable(options) {
        var isExpanding = false;
        var isCollapsing = false;
        var alreadyExpanded = false;
        var initiallyExpanded = Adform.getVar('initiallyExpanded') || 0;
        var offsetX;
        var offsetY;

        var settings = {
            expandEasing: (SingleExpanding && SingleExpanding.regularEaseOut) || "linear",
            collapseEasing: (SingleExpanding && SingleExpanding.regularEaseIn) || "linear",
            expandTime: Adform.getVar("expandTime") || "0",
            collapseTime: Adform.getVar("collapseTime") || "0",
            clicktag: null,
            target: null,
            video: null
        };

        function isFirstAutoExpand() {
            if (_AdformContent.options.autoShow && !alreadyExpanded) {
                return (alreadyExpanded = !alreadyExpanded);
            }
        }

        function setup(options, settings) {
            var prop;
            var prop2;
            for (prop in options) {
                if (options.hasOwnProperty(prop)) {
                    if (settings[prop] instanceof Object) {
                        for (prop2 in options[prop]) {
                            if (options[prop].hasOwnProperty(prop2)) {
                                settings[prop][prop2] = options[prop][prop2];
                            }
                        }
                    } else {
                        settings[prop] = options[prop];
                    }
                }
            }
        }

        function addEvents() {
            var vid = document.getElementById("video"); 
            
            if (settings.collapsed && settings.banner) {
                settings.collapsed.addEventListener("click", function () {
                    expandBanner();
                });
            }

            if (settings.expanded && settings.clickArea) {
                settings.clickArea.addEventListener("click", function () {
                    land(settings);
                });
            }

            if (settings.closeButton && settings.banner) {
                settings.closeButton.addEventListener("click", function (event) {
                    event.preventDefault();
                    event.stopPropagation();
                    dhtml.external.close && dhtml.external.close();
                    SingleExpanding || collapseBanner();
                });
            }

            SingleExpanding && SingleExpanding.on(SingleExpanding.COLLAPSE_END, function () {
                if (isCollapsing) {
                    toggleClass(settings.banner, ['adf-Transition', 'adf-FullyCollapsed']);
                    isCollapsing = false;
                }
            });

            SingleExpanding && SingleExpanding.on(SingleExpanding.COLLAPSE_START, function () {
                collapseBanner();
            });

            SingleExpanding && SingleExpanding.on(SingleExpanding.EXPAND_END, function () {
                if (isExpanding) {
                    toggleClass(settings.banner, ['adf-Transition', 'adf-FullyExpanded']);
                    isExpanding = false;
                }
                
            });

            SingleExpanding && SingleExpanding.on(SingleExpanding.EXPAND_UPDATE, function () {
                if (!isExpanding) {
                    expandBanner();
                    
                }
            });

            SingleExpanding && SingleExpanding.on(SingleExpanding.EXPAND_START, function () {
                expandBanner();
                
                vid.currentTime = 0;
                vid.play(); 
                
            });
        }

        function expandBanner() {
            if (isExpanding) return;
            isExpanding = true;
            isCollapsing = false;
            settings.banner.style.transition = "all " + settings.expandTime + "s ease-out";
            settings.banner.style.height = dhtml.height + "px";
            settings.banner.style.width = dhtml.width + "px";
            settings.banner.style.left = 0;
            settings.banner.style.top = 0;
            toggleClass(settings.banner, ['adf-Transition', 'adf-FullyCollapsed']);
            if (!isFirstAutoExpand()) { // prevent autocollapse from failing
                dhtml.external.expand && dhtml.external.expand(); // expanding banner, SingleExpanding.on(SingleExpanding.EXPAND_START... will be called here.
            }
            SingleExpanding || alternativeExpandEnd();
        }

        function collapseBanner() {
            if (isCollapsing) return;
            isCollapsing = true;
            isExpanding = false;
            settings.banner.style.transition = "all " + settings.collapseTime + "s ease-out";
            settings.banner.style.height = dhtml.collapsedHeight + "px";
            settings.banner.style.width = dhtml.collapsedWidth + "px";
            settings.banner.style.left = offsetX + "px";
            settings.banner.style.top = offsetY + "px";
            toggleClass(settings.banner, ['adf-Transition', 'adf-FullyExpanded']);
            SingleExpanding || alternativeCollapseEnd();
        }

        function alternativeExpandEnd() {
            if (!SingleExpanding) {
                setTimeout(function () {
                    toggleClass(settings.banner, ['adf-Transition', 'adf-FullyExpanded']);
                }, settings.expandTime * 1000);
                isExpanding = false;
            }
        }

        function alternativeCollapseEnd() {
            if (!SingleExpanding) {
                setTimeout(function () {
                    toggleClass(settings.banner, ['adf-FullyCollapsed', 'adf-Transition']);
                }, settings.collapseTime * 1000);
                isCollapsing = false;
            }
        }

        function setBG() {
            if (settings.collapsedBackground) {
                setBackgroundImage(settings.collapsed, settings.collapsedBackground);
                removeElems(document.querySelector('#adf-logo-collapsed'), document.querySelector('#adf-info-collapsed'));
                removeClass(settings.collapsed, ['adf-Background', 'adf-Border']);
            }
            if (settings.expandedBackground) {
                setBackgroundImage(settings.expanded, settings.expandedBackground);
                removeElems(document.querySelector('#adf-logo-expanded'), document.querySelector('#adf-info-expanded'));
                removeClass(settings.expanded, ['adf-Background', 'adf-Border']);
            }
        }

        this.init = function () {
            setup(options, settings);
            addClass(settings.banner, 'adf-FullyCollapsed');
            SingleExpanding && SingleExpanding.init(settings);
            offsetX = SingleExpanding ? dhtml.offsetX : ((_Adform.Data.expPosition.indexOf('left') >= 0) ? dhtml.width - dhtml.collapsedWidth : 0);
            offsetY = SingleExpanding ? dhtml.offsetY : ((_Adform.Data.expPosition.indexOf('up') >= 0) ? dhtml.height - dhtml.collapsedHeight : 0);
            if (!initiallyExpanded && settings.banner) {
                settings.banner.style.height = dhtml.collapsedHeight + "px";
                settings.banner.style.width = dhtml.collapsedWidth + "px";
                settings.banner.style.left = offsetX + "px";
                settings.banner.style.top = offsetY + "px";
            }
            setBG();
            addEvents();
        };

        this.expand = function () {
            dhtml.external.expand && dhtml.external.expand();
            SingleExpanding || expandBanner();
        };

        this.collapse = function () {
            dhtml.external.close && dhtml.external.close();
            SingleExpanding || collapseBanner();
        };

        this.SingleExpanding = SingleExpanding;
    }

    Adf.Expandable = Expandable;

}(Adf = window.Adf || {}));
